using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Grouping;

namespace GridException
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        protected override void OnLoad(EventArgs e)
        {
            DataTable dt1 = new DataTable("Test 1");
            DataColumn col1A = new DataColumn("A");
            col1A.AllowDBNull = false;
            col1A.DataType = typeof(double);
            DataColumn col1B = new DataColumn("B");
            col1B.AllowDBNull = false;
            DataColumn col1C = new DataColumn("C");
            col1C.AllowDBNull = false;

            DataTable dt2 = new DataTable("Test 2");
            DataColumn col2A = new DataColumn("A");
            col2A.AllowDBNull = false;
            DataColumn col2B = new DataColumn("B");
            col2B.AllowDBNull = false;
            DataColumn col2C = new DataColumn("C");
            col2C.AllowDBNull = false;
            DataColumn col2D = new DataColumn("D");
            col2D.AllowDBNull = false;

            dt1.Columns.Add(col1A);
            dt1.Columns.Add(col1B);
            dt1.Columns.Add(col1C);

            dt2.Columns.Add(col2A);
            dt2.Columns.Add(col2B);
            dt2.Columns.Add(col2C);
            dt2.Columns.Add(col2D);

            DataSet myDS = new DataSet();
            myDS.Tables.AddRange(new DataTable[] { dt1, dt2 });


            gridGroupingControl1.DataSource = myDS;

			this.gridGroupingControl1.TableControlCurrentCellControlKeyMessage += new GridTableControlCurrentCellControlKeyMessageEventHandler(gridGroupingControl1_TableControlCurrentCellControlKeyMessage);
			this.gridGroupingControl1.CurrentRecordContextChange += new Syncfusion.Grouping.CurrentRecordContextChangeEventHandler(gridGroupingControl1_CurrentRecordContextChange);
			this.gridGroupingControl1.TableControlCurrentCellKeyDown += new GridTableControlKeyEventHandler(gridGroupingControl1_TableControlCurrentCellKeyDown);
            this.gridGroupingControl1.TableControlCurrentCellValidating += new GridTableControlCancelEventHandler(gridGroupingControl1_TableControlCurrentCellValidating);
        }

		void gridGroupingControl1_TableControlCurrentCellKeyDown(object sender, GridTableControlKeyEventArgs e)
		{
			if (e.Inner.KeyCode == Keys.Up || e.Inner.KeyCode == Keys.Down)
			{
				GridCurrentCell cc = e.TableControl.CurrentCell;
				GridTableCellStyleInfo style = e.TableControl.GetTableViewStyleInfo(cc.RowIndex, cc.ColIndex);
				Element rec = style.TableCellIdentity.DisplayElement;
				if (rec != null)
				{
					DataRowView drv = rec.GetData() as DataRowView;
					string field = "";
					if (drv != null && cc.ConfirmChanges() && !this.ValidateDataRow(drv.Row, out field))
					{
						e.Inner.Handled = true;
						if (rec is AddNewRecord)
						{
							((AddNewRecord)rec).SetCurrent(field);
						}
						else if (rec is GridRecord)
						{
							((GridRecord)rec).SetCurrent(field);
						}
					}
					
				}
			}
		}


        void gridGroupingControl1_TableControlCurrentCellValidating(object sender, GridTableControlCancelEventArgs e)
        {
            GridCurrentCell cc = e.TableControl.CurrentCell;
            GridTableCellStyleInfo style = e.TableControl.GetTableViewStyleInfo(cc.RowIndex, cc.ColIndex);
            GridColumnDescriptor cd = style.TableCellIdentity.Column;

            string ll = e.TableControl.CurrentCell.Renderer.ControlText;
            string tableName = e.TableControl.TableDescriptor.Name;

            DataTable actTable = (gridGroupingControl1.DataSource as DataSet).Tables[tableName];
            Type actType = actTable.Columns[cd.Name].DataType;
            try
            {
                Convert.ChangeType(ll, actType);
            }
            catch (Exception err)
            {
                MessageBox.Show("Wrong value for that type");
                e.Inner.Cancel = true;
            }
        }
		

		void gridGroupingControl1_TableControlCurrentCellControlKeyMessage(object sender, GridTableControlCurrentCellControlKeyMessageEventArgs e)
		{
			Keys keyCode = (Keys)((int)e.Inner.Msg.WParam) & Keys.KeyCode;
			if (keyCode == Keys.Enter)
			{
				if (e.Inner.Msg.Msg == 0x100) //KEYDOWN
				{
					SendKeys.Send("{TAB}");
				}
				e.Inner.Handled = true;
			}
			
		}

		
		void gridGroupingControl1_CurrentRecordContextChange(object sender, Syncfusion.Grouping.CurrentRecordContextChangeEventArgs e)
		{
			if (e.Action == CurrentRecordAction.EndEditCalled && e.Record != null)
			{
				string field = "";
				DataRowView drv = (e.Record != null) ? e.Record.GetData() as DataRowView : null;
				if (drv != null && !ValidateDataRow(drv.Row, out field))
				{
					e.Cancel = true;

					if (e.Record is AddNewRecord)
					{
						((AddNewRecord)e.Record).SetCurrent(field);
					}
					else if (e.Record is GridRecord)
					{
						((GridRecord)e.Record).SetCurrent(field);
					}
				}
				
			}
		}

		private bool ValidateDataRow(DataRow dr, out string field)
		{
			field = "";
			bool b = true;

			DataTable dt = dr.Table;
			foreach (DataColumn dc in dt.Columns)
			{
				if (!dc.AllowDBNull && (dr[dc] == null || dr[dc] == DBNull.Value))
				{
					b = false;
					field = dc.ColumnName;
					MessageBox.Show(string.Format("nulls not allowed in {0}", field));
					break;
				}
			}
			return b;
			
		}
    }
}